#include "helper_caller.h"

HelperCaller::HelperCaller(QObject *parent = 0)
    : QObject(parent)
{
    callThread = new CallThread(this);
    connect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(emitResult(QStringList)));
    process = new QProcess(this);
    process->setReadChannelMode(QProcess::MergedChannels);
    connect(process, SIGNAL(readyRead()), this, SLOT(errorQEMU()));
}
HelperCaller::~HelperCaller()
{
    disconnect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(emitResult(QStringList)));
    disconnect(process, SIGNAL(readyRead()), this, SLOT(errorQEMU()));
    if (callThread) callThread->stop();
    delete callThread;
    callThread = 0;
    delete process;
    process = 0;
}
bool HelperCaller::actionActivated(const QStringList &args)
{
    return actionActivated(args, 60);
}
bool HelperCaller::actionActivated(const QStringList &args, int timeout = 60)
{
    if (args.isEmpty()) return false;
    //qDebug() << "pretending to be the mechanism for action:" << args.first();
    if ( isQEMUcontainer(args) ) {
        return runQEMUcontainer(args, timeout);
    };
    // For LXC container continue:
    // As debug message says we are pretending to be the mechanism for the
    // following action, here you will actually call your DBus helper that
    // will run as root (setuid is not needed, see DBus docs).
    // In the helper application you will issue checkAuthorizationSync,
    // passing the action id and the caller pid (which DBus will tell you).

    Authority::Result result;
    UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));

    result = Authority::instance()->checkAuthorizationSync(QString("%1.%2").arg(DOMAIN).arg(args.first()), subject,
             Authority::AllowUserInteraction);
    bool auth = (result == Authority::Yes);
    //qDebug() << "caller is"<< (( auth )? "" : "not") << "authorized to do:" << args.first();
    if ( auth ) {
        QStringList _args = args;
        if ( !callThread->isRunning() ) {
            callThread->setArgs(_args, timeout);
            callThread->start();
        } else addMsgToLog("Authoring", "Process is busy.\nTry later.");
    }
    else addMsgToLog("Authoring",
        QString("Not authorized to do %1.").arg(args.first()));
    return auth;
}
void HelperCaller::emitResult(QStringList data)
{
    QStringList _result = data;
    //qDebug()<<_result<<"result";
    if ( callThread->isRunning() ) callThread->stop();
    emit resultData(_result);
}
bool HelperCaller::isQEMUcontainer(QStringList args)
{
    QString QEMU = QString("qemu:///session");
    int i = args.indexOf("-c");
    return ( args.contains(QEMU) && (i+1) && args.at(i+1)==QEMU );
}
bool HelperCaller::runQEMUcontainer(QStringList args, int timeout = 60)
{
    bool state = false;
    //qDebug()<<args<<"QEMU";
    QStringList env;
    QStringList Args;
    int separator = args.lastIndexOf("ENVIRONMENT:");
    if ( separator+1 ) {
        foreach (QString var, args) {
            if ( args.indexOf(var) >= separator ) {
                env.append(var);
            }
            else {
                Args.append(var);
            };
        };
    };
    if ( args.first()=="getProcessState" ) {
        state = ( process->state()==QProcess::Running );
    }
    else if ( args.first()=="start" ) {
        if ( process->state()==QProcess::NotRunning ) {
            QStringList setMode = QStringList()<<"setCurrentSELinuxMode"<<"0";
            if ( actionActivated(setMode, 60) ) {
                Args.removeFirst();  // remove method name
                QString runApp = Args.first();
                Args.removeFirst();
                process->setEnvironment(env);
                process->start(runApp, Args);
                state = process->waitForStarted(timeout*1000);
            };
        }
        else {
            addMsgToLog("Authoring", "Process is busy.\nTry later.");
            state = false;
        };
    }
    else if ( args.first()=="stop" ) {
        if ( process->state()==QProcess::Running|QProcess::Starting ) {
            process->kill();
            //qDebug()<<Args<<"killed";
        };
        state = ( process->state()==QProcess::NotRunning );
    };
    emitResult( QStringList()<<QVariant(state).toString()<<args.first()<<QString::number( process->pid() ) );
    return state;
}
void HelperCaller::errorQEMU()
{
    QByteArray _errors;
    _errors.append( process->readAllStandardOutput() );
    _errors.append("\n");
    _errors.append( process->readAllStandardError() );
    QTextStream stream(&_errors);
    QString errors = stream.readAll();
    qDebug()<<"************"<<errors;
}
void HelperCaller::stopThread()
{
    callThread->stop();
}
void HelperCaller::addMsgToLog(QString title, QString msg)
{
    QString time = QTime::currentTime().toString();
    QString errorMsg = QString("<b>%1 %2:</b><br>%3").arg(time).arg(title).arg(msg);
    emit warning(errorMsg);
}
