#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
  restoreGeometry(settings.value("Geometry").toByteArray());
  setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
  setMinimumSize(100, 100);
  setContentsMargins(0, 0, 0, 5);
  setWindowTitle("Virt-Sandbox Runner");
  QStringList searchThemePath;
  searchThemePath.append(QIcon::themeSearchPaths());
  searchThemePath.insert(0, "/usr/share/sandbox-runner-data/");
  QIcon::setThemeSearchPaths(searchThemePath);
  QIcon::setThemeName("icons");
  setWindowIcon(QIcon::fromTheme("applications-safety-virtual"));
  setMouseTracking(true);
  initTrayIcon();
  initJobWidget();
  initToolBar();
  initDockWidget();
  timerId = startTimer(1000);
}
MainWindow::~MainWindow()
{
  disconnect(docContent, SIGNAL(stateChanged(const QString&)), this, SLOT(selinuxStateChanged(const QString&)));
  disconnect(docContent, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  disconnect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, \
                    SLOT(trayIconActivated(QSystemTrayIcon::ActivationReason)));
  disconnect(trayIcon->hideAction, SIGNAL(triggered()), this, SLOT(changeVisibility()));
  disconnect(trayIcon->closeAction, SIGNAL(triggered()), this, SLOT(closeEvent()));
  disconnect(jobWidget, SIGNAL(removeJob(QString &)), this, SLOT(removeJobItem(QString &)));
  disconnect(jobWidget, SIGNAL(messageShowed()), this, SLOT(mainWindowUp()));
  disconnect(jobWidget, SIGNAL(showVirtNetControl()), this, SLOT(showVirtNetDialog()));
  disconnect(jobWidget, SIGNAL(warning(QString&)), this, SLOT(writeToErrorLog(QString&)));
  disconnect(toolBar->_hideAction, SIGNAL(triggered()), this, SLOT(changeVisibility()));
  disconnect(toolBar->_createAction, SIGNAL(triggered()), this, SLOT(createNewJobItem()));
  disconnect(toolBar->_editAction, SIGNAL(triggered()), this, SLOT(editCurrentJobItem()));
  disconnect(toolBar->_deleteAction, SIGNAL(triggered()), this, SLOT(deleteCurrentJobItem()));
  disconnect(toolBar->_runAction, SIGNAL(triggered()), this, SLOT(runCurrentJob()));
  disconnect(toolBar->_stopAction, SIGNAL(triggered()), this, SLOT(stopCurrentJob()));
  disconnect(toolBar->_stopAllAction, SIGNAL(triggered()), this, SLOT(stopAllJob()));
  disconnect(toolBar->_virtNetAction, SIGNAL(triggered()), this, SLOT(closeVirtNetDialog()));
  disconnect(toolBar->_exitAction, SIGNAL(triggered()), this, SLOT(closeEvent()));
  disconnect(toolBar, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));

  if ( wait_thread!=NULL ) {
      disconnect(wait_thread, SIGNAL(finished()), this, SLOT(closeEvent()));
      delete wait_thread;
      wait_thread = 0;
  };

  if ( virtNetControl!=NULL ) {
    delete virtNetControl;
    virtNetControl = 0;
  };

  delete docContent;
  docContent = 0;
  delete doc;
  doc = 0;
  delete trayIcon;
  trayIcon = 0;
  delete jobWidget;
  jobWidget = 0;
  delete toolBar;
  toolBar = 0;
}
void MainWindow::closeEvent(QCloseEvent *ev)
{
  settings.setValue("Geometry", saveGeometry());
  settings.setValue("ToolBarArea", toolBarArea(toolBar));
  settings.beginGroup("JobListColumns");
  settings.setValue("column0", jobWidget->columnWidth(0));
  settings.setValue("column1", jobWidget->columnWidth(1));
  settings.setValue("column2", jobWidget->columnWidth(2));
  settings.endGroup();
  settings.sync();
  if (timerId) {
      killTimer(timerId);
      timerId = 0;
  };
  if ( !this->isVisible() ) changeVisibility();
  if ( runningJobsExist() && wait_thread==NULL ) {
      QString q;
      q.append("Running Jobs are exist.\nKill it at exit?");
      int answer = QMessageBox::question(this, "Action", q, QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);
      if ( answer == QMessageBox::Cancel ) {
          ev->ignore();
          return;
      };
      jobWidget->setEnabled(false);
      toolBar->setEnabled(false);
      wait_thread = new Wait(this);
      connect(wait_thread, SIGNAL(finished()), this, SLOT(closeEvent()));
      wait_thread->setWdgReference(jobWidget);
      wait_thread->setMode( answer == QMessageBox::Yes );
      wait_thread->start();
      ev->ignore();
  } else if ( !runningJobsExist() && (wait_thread==NULL || wait_thread->isFinished()) ) {
      ev->accept();
  } else {
      ev->ignore();
  };
}
void MainWindow::closeEvent()
{
  this->close();
}
void MainWindow::initTrayIcon()
{
  trayIcon = new TrayIcon(this);
  connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, \
  SLOT(trayIconActivated(QSystemTrayIcon::ActivationReason)));
  connect(trayIcon->hideAction, SIGNAL(triggered()), this, SLOT(changeVisibility()));
  connect(trayIcon->closeAction, SIGNAL(triggered()), this, SLOT(closeEvent()));
}
void MainWindow::mainWindowUp()
{
    if ( !this->isVisible() ) {
        changeVisibility();
        QTimer::singleShot(333, this, SLOT(autoHide()));
    };
}
void MainWindow::changeVisibility()
{
  if (this->isVisible()) {
      this->hide();
      trayIcon->hideAction->setText (QString("Up"));
      trayIcon->hideAction->setIcon (QIcon::fromTheme("up"));
  } else {
      this->show();
      trayIcon->hideAction->setText (QString("Down"));
      trayIcon->hideAction->setIcon (QIcon::fromTheme("down"));
  };
}
void MainWindow::trayIconActivated(QSystemTrayIcon::ActivationReason r)
{
  if (r==QSystemTrayIcon::Trigger) changeVisibility();
}
void MainWindow::initJobWidget()
{
  jobWidget= new JobList(this);
  settings.beginGroup("JobListColumns");
  jobWidget->setColumnWidth(0, settings.value("column0", 132).toInt());
  jobWidget->setColumnWidth(1, settings.value("column1", 32).toInt());
  jobWidget->setColumnWidth(2, settings.value("column2", 32).toInt());
  settings.endGroup();
  settings.beginGroup("Tasks");
  QStringList groups = settings.childGroups();
  settings.endGroup();
  QList<QString>::const_iterator i;
  for (i=groups.constBegin(); i!=groups.constEnd(); ++i) {
      //QString s = (*i);
      jobWidget->addJobItem(*i);
  };
  setCentralWidget(jobWidget);
  connect(jobWidget, SIGNAL(removeJob(QString &)), this, SLOT(removeJobItem(QString &)));
  connect(jobWidget, SIGNAL(messageShowed()), this, SLOT(mainWindowUp()));
  connect(jobWidget, SIGNAL(showVirtNetControl()), this, SLOT(closeVirtNetDialog()));
  connect(jobWidget, SIGNAL(warning(QString&)), this, SLOT(writeToErrorLog(QString&)));
}
void MainWindow::initToolBar()
{
  toolBar = new ToolBar(this);
  connect(toolBar->_hideAction, SIGNAL(triggered()), this, SLOT(changeVisibility()));
  connect(toolBar->_createAction, SIGNAL(triggered()), this, SLOT(createNewJobItem()));
  connect(toolBar->_editAction, SIGNAL(triggered()), this, SLOT(editCurrentJobItem()));
  connect(toolBar->_deleteAction, SIGNAL(triggered()), this, SLOT(deleteCurrentJobItem()));
  connect(toolBar->_runAction, SIGNAL(triggered()), this, SLOT(runCurrentJob()));
  connect(toolBar->_undockAction, SIGNAL(triggered()), this, SLOT(undockCurrentJob()));
  connect(toolBar->_stopAction, SIGNAL(triggered()), this, SLOT(stopCurrentJob()));
  connect(toolBar->_stopAllAction, SIGNAL(triggered()), this, SLOT(stopAllJob()));
  connect(toolBar->_virtNetAction, SIGNAL(triggered()), this, SLOT(showVirtNetDialog()));
  connect(toolBar->_exitAction, SIGNAL(triggered()), this, SLOT(closeEvent()));
  connect(toolBar, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  int area_int = settings.value("ToolBarArea", 4).toInt();
  this->addToolBar(toolBar->get_ToolBarArea(area_int), toolBar);
}
void MainWindow::initDockWidget()
{
    doc = new QDockWidget(this);
    doc->setWindowTitle("SELinux Status / Error Log");
    doc->setFeatures(QDockWidget::NoDockWidgetFeatures);
    doc->hide();
    docContent = new DocContent(doc);
    docContent->setInitContent(toolBar->iconSize());
    doc->setWidget(docContent);
    addDockWidget(Qt::BottomDockWidgetArea, doc);
    connect(docContent, SIGNAL(stateChanged(const QString&)), this, SLOT(selinuxStateChanged(const QString&)));
    connect(docContent, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
}
void MainWindow::editCurrentJobItem()
{
  jobWidget->editItemAction();
}
void MainWindow::createNewJobItem()
{
  QString s = QString("<noname>");
  jobWidget->addJobItem(s);
}
void MainWindow::deleteCurrentJobItem()
{
  jobWidget->deleteCurrentJobItem();
}
void MainWindow::removeJobItem(QString &job)
{
  settings.beginGroup("Tasks");
  settings.remove(job);
  settings.endGroup();
  QStringList fullPath = settings.fileName().split(QDir::separator());
  fullPath.removeLast();
  fullPath.append(QString("%1.included").arg(job));
  QString _fn = fullPath.join(QDir::separator());
  QFile::remove(_fn);
  //qDebug()<<job<<"job deleted";
}
void MainWindow::runCurrentJob()
{
    QModelIndex _item = jobWidget->currentIndex();
    if (_item.isValid()) {
        jobWidget->runJob(_item);
    };
}
void MainWindow::undockCurrentJob()
{
    QModelIndex _item = jobWidget->currentIndex();
    if (_item.isValid()) {
        jobWidget->undockJob(_item);
    };
}
void MainWindow::stopCurrentJob()
{
    QModelIndex _item = jobWidget->currentIndex();
    if (_item.isValid()) {
        jobWidget->stopJob(_item);
    };
}
void MainWindow::stopAllJob()
{
  int count = jobWidget->jobItemModel->rowCount();
  for (int i = 0; i< count; i++) stopJob(i);
}
void MainWindow::stopJob(int i)
{
  //qDebug()<<i<<" item to stop";
  QModelIndex _item = jobWidget->jobItemModel->index(i, 0);
  jobWidget->stopJob(_item);
}
bool MainWindow::runningJobsExist()
{
    bool result = false;
    int count = jobWidget->jobItemModel->rowCount();
    for (int i=0; i<count; i++) {
        JobItemIndex *item = jobWidget->jobItemModel->jobItemDataList.at(i);
        //qDebug()<<jobWidget->item(i)->text()<< jobWidget->item(i)->data(Qt::UserRole).toMap().value("isRunning").toBool();
        if ( item->getData().value("isRunning").toBool() ||
            !item->getData().value("availability").toBool() ) {
            result = true;
            break;
        };
    };
    return result;
}
void MainWindow::mouseMoveEvent(QMouseEvent *ev)
{
    if ( ev->type()==QEvent::MouseMove ) showDoc();
}
void MainWindow::timerEvent(QTimerEvent *event)
{
    int _timerId = event->timerId();
    if ( _timerId && timerId==_timerId ) {
        int rc;
        rc = is_selinux_enabled();
        if (rc < 0) return;
        if (rc == 1) {
            rc = security_getenforce();
            if (rc < 0) return;
            if (rc)
                docContent->setDarkSEIcon("Enforcing");
            else
                docContent->setLightSEIcon("Permissive");
        } else {
            docContent->setLightSEIcon("Disabled");
        };
        if ( initialSELinuxState.isEmpty() || initialSELinuxState.isNull() )
            initialSELinuxState.append(docContent->getSELinuxState());
    };
}
void MainWindow::showDoc()
{
    doc->show();
    QTimer::singleShot(3000, doc, SLOT(hide()));
}
void MainWindow::selinuxStateChanged(const QString &state)
{
    trayIcon->showMessage(windowTitle(), QString("SELinux is %1").arg(state), QSystemTrayIcon::Warning, 3000);
    showDoc();
}
bool MainWindow::restoreSELinuxState()
{
    bool result = true;
    if ( initialSELinuxState != "Disabled" ) {
        QString currentSEState = docContent->getSELinuxState();
        if ( initialSELinuxState != currentSEState ) {
            QString q = QString("Initial SELinuxState is changed:\n(%1 -> %2).\nRestore it?")
                           .arg(initialSELinuxState).arg(currentSEState);
            int answer = QMessageBox::question(this,
                "Restore SELinux State", q, QMessageBox::Ok, QMessageBox::Cancel);
            if ( answer == QMessageBox::Ok ) {
                /* Restore SELinux state */
                Authority::Result _result;
                UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));
                QStringList args;
                args << "setCurrentSELinuxMode" << QString(( initialSELinuxState=="Enforcing" ) ? "1" : "0");
                _result = Authority::instance()->checkAuthorizationSync(QString("%1.%2").arg(DOMAIN).arg(args.first()), subject,
                         Authority::AllowUserInteraction);
                bool auth = (_result == Authority::Yes);
                if ( auth ) {
                    QStringList parameters;
                    parameters<<args;
                    parameters.removeFirst();
                    QDBusMessage message;
                    message = QDBusMessage::createMethodCall(DOMAIN, "/", DOMAIN, args.first().toLatin1());
                    QList<QVariant> argumentList;
                    argumentList << qVariantFromValue(parameters);
                    message.setArguments(argumentList);
                    // notice the systemBus here..
                    /* used QDBus::Block call mode for wait reply */
                    QDBusMessage reply = QDBusConnection::systemBus().call(message, QDBus::Block);
                    //qDebug()<<reply.arguments()<<"reply<<"<<args.first();
                    if ( reply.type() != QDBusMessage::ReplyMessage ) result = false;
                    if ( reply.arguments().size() ) {
                        //qDebug()<<reply.arguments();
                        QString msg;
                        int rc = security_getenforce();
                        if (rc < 0) {
                            msg = "SELinux isn't restored.";
                            result = false;
                        }
                        else if (rc) {
                            msg = QString("SELinux restored to Enforcing mode.");
                        }
                        else {
                            msg = QString("SELinux restored to Permissive mode.");
                        };
                        QMessageBox::information(this, "Restore SELinux State", msg);
                    }
                    else result = false;
                }
                else result = false;
            };
        };
    };
    return result;
}
void MainWindow::autoHide()
{
    if (this->isVisible()) changeVisibility();
}
void MainWindow::showVirtNetDialog()
{
    Authority::Result result;
    UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));

    result = Authority::instance()->checkAuthorizationSync(
             QString("%1.%2").arg(DOMAIN).arg("getNetworkControlAccess"), subject,
             Authority::AllowUserInteraction);
    bool auth = (result == Authority::Yes);
    if (auth) {
        if ( virtNetControl==NULL ) virtNetControl = new VirtNetControl(this);
        if ( virtNetControl->isHidden() ) {
            virtNetControl->show();
            virtNetControl->startUpdateTimer();
        };
        //virtNetControl->move(mapToGlobal(this->pos()));
    } else QMessageBox::information(this, "Access", "Access denied");
}
void MainWindow::closeVirtNetDialog()
{
    if ( virtNetControl!=NULL && virtNetControl->isVisible() )
        virtNetControl->close();
}
void MainWindow::writeToErrorLog(QString &msg)
{
    docContent->appendErrorMsg(msg);
}
