#include "directory_settings.h"

/* -m TYPE:DST=SRC */

#define TITLE QString("<b>DESCRIPTION:<br>-m TYPE:DST=SRC, --mount TYPE:DST=SRC<br></b>")
#define COMMENT1 QString("If TYPE is host-bind, then SRC is interpreted as<br>the path to a directory on the host filesystem.<br>If \"SRC\" is the empty string,<br>then a temporary (empty) directory is created<br>on the host before starting the sandbox<br>and deleted afterwards.<br>The \"--include\" option is useful for populating<br>these temporary directories with copies of host files.")
#define COMMENT2 QString("If TYPE is host-image, then SRC is interpreted as<br>the path to a disk image file on the host filesystem.<br>The image should be formatted with a filesystem<br>that can be auto-detected by the sandbox,<br>such as ext3, ext4, etc.<br>The disk image itself should be a raw file,<br>not qcow2 or any other special format.")
#define COMMENT3 QString("If TYPE is guest-bind,<br>then SRC is interpreted as the path<br>to another directory in the container filesystem.")
#define COMMENT4 QString("If TYPE is ram, then SRC is interpreted<br>as specifying the size of the RAM disk in bytes.<br>The suffix K, KiB, M, MiB, G, GiB can used<br>to alter the units from bytes to a coarser level.")

#define HOST_BIND QString("host-bind")
#define HOST_IMAGE QString("host-image")
#define GUEST_BIND QString("guest-bind")
#define RAM QString("ram")

MountSet::MountSet(QWidget *parent) :
    QWidget(parent)
{
    setContentsMargins(1, 1, 1, 1);
    fileName = QString();
    commonLayout = new QVBoxLayout();
    useMount = new QCheckBox("Sets up a mount inside the sandbox\nat DST backed by SRC", this);
    useMount->setChecked(false);

    type = new QComboBox(this);
    type->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    type->setToolTip("Mount Type");
    type->setEnabled(false);
    QIcon info = QIcon::fromTheme("dialog-information");
    comment = new QPushButton(info, "", this);
    comment->setToolTip("Description");
    comment->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    comment->setEnabled(false);
    titleLabel = new QLabel("<b>DST=SRC:</b>", this);
    titleLabel->setAlignment(Qt::AlignRight);
    titleLabel->setEnabled(false);
    addRule = new QPushButton(QIcon::fromTheme("list-add"), "", this);
    addRule->setToolTip("Add mount rule");
    addRule->setEnabled(false);
    delRule = new QPushButton(QIcon::fromTheme("list-remove"), "", this);
    delRule->setToolTip("Delete mount rule");
    delRule->setEnabled(false);
    addPath = new QPushButton(QIcon::fromTheme("edit-find"), "", this);
    addPath->setToolTip("Add mounted SRC");
    addPath->setEnabled(false);

    layout1 = new QHBoxLayout();
    layout1->addWidget(type);
    layout1->addWidget(comment);
    layout2 = new QHBoxLayout();
    layout2->addWidget(addRule, Qt::AlignLeft);
    layout2->addWidget(delRule, Qt::AlignLeft);
    layout2->addWidget(titleLabel, Qt::AlignRight);
    layout2->addWidget(addPath);

    bindRule = new QLineEdit(this);
    bindRule->setToolTip("Enter mount bind or image");
    bindRule->setEnabled(false);
    to_mount = new QListWidget(this);
    to_mount->setDragDropMode(QAbstractItemView::InternalMove);
    to_mount->setSelectionMode(QAbstractItemView::MultiSelection);
    to_mount->setEnabled(false);

    commonLayout->addWidget(useMount, Qt::AlignLeft);
    commonLayout->insertLayout(1, layout1);
    commonLayout->insertLayout(2, layout2);
    commonLayout->addWidget(bindRule);
    commonLayout->addWidget(to_mount);
    commonLayout->insertStretch(-1);
    setLayout(commonLayout);
    QStringList types;
    types<<HOST_BIND<<HOST_IMAGE<<GUEST_BIND<<RAM;
    foreach (QString var, types) {
        type->addItem(var);
    };
    changeComment(types.first());
    connect(useMount, SIGNAL(toggled(bool)), this, SLOT(changeMountState(bool)));
    connect(comment, SIGNAL(clicked()), this, SLOT(showDescription()));
    connect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    connect(addRule, SIGNAL(clicked()), this, SLOT(addRuleToList()));
    connect(delRule, SIGNAL(clicked()), this, SLOT(delRuleFromList()));
    connect(bindRule, SIGNAL(textChanged(QString)), this, SLOT(changeMountRule(QString)));
    connect(bindRule, SIGNAL(returnPressed()), this,  SLOT(addRuleToList()));
    connect(addPath, SIGNAL(clicked()), this, SLOT(addPathToRule()));
    connect(to_mount, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editMountItem(QListWidgetItem*)));
}
MountSet::~MountSet()
{
    disconnect(useMount, SIGNAL(toggled(bool)), this, SLOT(changeMountState(bool)));
    disconnect(comment, SIGNAL(clicked()), this, SLOT(showDescription()));
    disconnect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    disconnect(addRule, SIGNAL(clicked()), this, SLOT(addRuleToList()));
    disconnect(delRule, SIGNAL(clicked()), this, SLOT(delRuleFromList()));
    disconnect(bindRule, SIGNAL(textChanged(QString)), this, SLOT(changeMountRule(QString)));
    disconnect(bindRule, SIGNAL(returnPressed()), this,  SLOT(addRuleToList()));
    disconnect(addPath, SIGNAL(clicked()), this, SLOT(addPathToRule()));
    disconnect(to_mount, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editMountItem(QListWidgetItem*)));
    delete useMount;
    useMount = 0;
    delete type;
    type = 0;
    delete comment;
    comment = 0;
    delete titleLabel;
    titleLabel = 0;
    delete addRule;
    addRule = 0;
    delete delRule;
    delRule = 0;
    delete addPath;
    addPath = 0;
    delete layout1;
    layout1 = 0;
    delete layout2;
    layout2 = 0;
    delete bindRule;
    bindRule = 0;
    delete to_mount;
    to_mount = 0;
    delete commonLayout;
    commonLayout = 0;
}
void MountSet::set_FileName(const QString s)
{
  fileName = s;
}
void MountSet::changeComment(QString s)
{
    addPath->setEnabled(true);
    if (s.isEmpty() || s.isNull()) return;
    if (s.toLower()==HOST_BIND) {
        description = QString("%1<br>%2").arg(TITLE).arg(COMMENT1);
        bindRule->setPlaceholderText("/tmp=/var/lib/sandbox/demo/tmp");
    }
    else if (s.toLower()==HOST_IMAGE) {
        description = QString("%1<br>%2").arg(TITLE).arg(COMMENT2);
        bindRule->setPlaceholderText("/=/var/lib/sandbox/demo.img");
    }
    else if (s.toLower()==GUEST_BIND) {
        description = QString("%1<br>%2").arg(TITLE).arg(COMMENT3);
        bindRule->setPlaceholderText("/home=/tmp/home");
    }
    else if (s.toLower()==RAM) {
        description = QString("%1<br>%2").arg(TITLE).arg(COMMENT4);
        bindRule->setPlaceholderText("/tmp=500M");
        addPath->setEnabled(false);
    };
    bindRule->setText(type->itemData(type->currentIndex()).toString());
}
void MountSet::setCurrentType(QString s)
{
    int i = type->findText(s);
    if (i+1) type->setCurrentIndex(i);
}
QString MountSet::getCurrentType() const
{
    return type->currentText();
}
void MountSet::changeMountState(bool b)
{
    type->setEnabled(b);
    comment->setEnabled(b);
    titleLabel->setEnabled(b);
    addRule->setEnabled(b);
    delRule->setEnabled(b);
    if (type->currentText()!=RAM) addPath->setEnabled(b);
    else addPath->setEnabled(false);
    bindRule->setEnabled(b);
    to_mount->setEnabled(b);
}
void MountSet::setCurrentRule(QString s)
{
    if ( s.isEmpty() ) return;
    fileName = s;
    QFile *f = new QFile(fileName);
    if (f->exists())
      {
        QStringList _list;
        QByteArray _data;
        f->open(QIODevice::ReadOnly|QIODevice::Text);
        _data = f->readAll();
        f->close();
        _list = QTextStream(&_data).readAll().split("\n");
        _list.removeAll("");
        to_mount->clear();
        to_mount->addItems(_list);
      }
    else QMessageBox::information(this, "Info", "MountFile not exist.");
    f->deleteLater();
}
QStringList MountSet::getCurrentRules() const
{
    QStringList rules;
    int count = to_mount->count();
    for (int i=0; i<count; i++) {
        rules.append(to_mount->item(i)->text());
    };
    return rules;
}
void MountSet::changeMountRule(const QString &s)
{
    if (s.isNull()) return;
    type->setItemData(type->currentIndex(), QVariant(s));
}
void MountSet::showDescription()
{
    QMessageBox::information(this, "Description", description);
}
void MountSet::addRuleToList()
{
    if ( !bindRule->text().isEmpty() ) {
        to_mount->addItem(QString("%1:%2").arg(type->currentText()).arg(bindRule->text()));
        //to_mount->sortItems();
        bindRule->clear();
    };
}
void MountSet::delRuleFromList()
{
    QList<QListWidgetItem *>::const_iterator i;
    QList<QListWidgetItem *> selected;
    selected = to_mount->selectedItems();
    for (i = selected.begin(); i!=selected.end(); i++) { delItem(*i); };
    selected.clear();
}
void MountSet::delItem(QListWidgetItem *_item)
{
    if (_item) {
        to_mount->takeItem(to_mount->row(_item));
    }
    else QMessageBox::information(this, "Info", "Item not exist.");
}
void MountSet::addPathToRule()
{
    QString path;
    if (type->currentText()==HOST_IMAGE) {
        path = QFileDialog::getOpenFileName(this, "Include files", "~", "Images (*.img);;");
    }
    else {
        path = QFileDialog::getExistingDirectory(this, "Include directory", "~");
    };
    bindRule->clear();
    bindRule->setText(path);
}
QString MountSet::get_FileName() const
{
    int l;
    QFile *f = new QFile(fileName);
    QString s = getCurrentRules().join("\n");
    f->open(QIODevice::WriteOnly|QIODevice::Text);
    l = f->write(s.toUtf8());
    f->close();
    f->deleteLater();
    return ( l == s.toUtf8().length() ) ? fileName : QString();
}
void MountSet::editMountItem(QListWidgetItem *item)
{
    to_mount->takeItem(to_mount->row(item));
    QString text = item->text();
    int idx = text.indexOf(":");
    int i = type->findText(text.left(idx));
    if (i+1) type->setCurrentIndex(i);
    bindRule->setText(text.mid(idx+1));
}
void MountSet::add_HOST_BIND_DST(QString &dst)
{
    useMount->setCheckState(Qt::Checked);
    to_mount->addItem(QString("%1:%2").arg(HOST_BIND).arg(dst));
}
